package cn.com.ty.lift.system.framework.aspect;

import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.utils.StrPool;
import cn.com.xwy.boot.web.dto.RestResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author wcz
 * @date 2019/12/10
 * @description 全局异常捕获处理
 */
@Slf4j
@RestControllerAdvice
public class GlobalDefaultExceptionHandler {

    private static final String classPrefix = "cn.com.ty.lift.system";

    @ExceptionHandler(value = ExceptionInInitializerError.class)
    @ResponseStatus(value = HttpStatus.BAD_REQUEST)
    public Object badRequest() {
        return RestResponse.exception("Bad Request");
    }

    @ExceptionHandler(value = Exception.class)
    public Object defaultErrorHandler(final HttpServletRequest req, final HttpServletResponse resp, final Exception error) {
        // 原始错误集合数据偏多，包含native错误
        final StackTraceElement[] stackTrace = error.getStackTrace();
        // 过滤错误信息
        final List<StackTraceElement> elements = Stream.of(stackTrace)
                .filter(st -> (st.getClassName().contains(classPrefix) && st.getFileName().contains(StrPool.DOT_JAVA)))
                .collect(Collectors.toCollection(LinkedList::new));

        final StackTraceElement[] dwStackTrace = new StackTraceElement[elements.size()];
        elements.toArray(dwStackTrace);

        // 填充StackTrace
        error.setStackTrace(dwStackTrace);
        log.error(StrPool.LOG_GLOBAL_EXCEPTION_LINE);
        log.error(StrPool.LOG_URL + req.getRequestURI());
        log.error(StrPool.LOG_STATUS + resp.getStatus());
        log.error(StrPool.LOG_EXCEPTION, error);

        if (error instanceof Exception) {
            return RestResponse.fail(ApiConstants.RESULT_EXCEPTION, "操作异常，请重试");
        }
        return RestResponse.exception();
    }
}
